--[[
	Scripts that add the "kommunal" mission type and loads the associated vehicles
	-- Authors: SMI Modding Team (Nicola)
    -- Game version: Farming Simulator 22
    -- Version: 1.1 - 3.07.2024	
]]

KommunalMission = {
	REWARD_PER_HA = 500,
	SUCCESS_FACTOR = 0.90,
}

local KommunalMission_mt = Class(KommunalMission, WeedMission)
InitObjectClass(KommunalMission, "KommunalMission")

function KommunalMission.new(isServer, isClient, customMt)
	local self = WeedMission.new(isServer, isClient, customMt or KommunalMission_mt)
	self.workAreaTypes = {
		[WorkAreaType.MULCHER] = true,
		[WorkAreaType.MOWER] = true
	}
	self.area_trimmed = 0
	self.reimbursementPerHa = 0
	self.area_tot = math.random()*500 + 1000
	self.rewardPerHa = KommunalMission.REWARD_PER_HA * self.area_tot/1250
	return self
end

function KommunalMission:loadFromXMLFile(xmlFile, key)
	-- se non ci sono le funzionalità di base
	if not KommunalMission:superClass().loadFromXMLFile(self, xmlFile, key) then
		return false
	end

	local field_key = key .. ".field"
	local kommunalKey = string.format("%s.kommunal", key)
	if hasXMLProperty(xmlFile, kommunalKey) then
		self.area_tot = getXMLFloat(xmlFile, kommunalKey .. "#areaToCut")
		self.area_trimmed = getXMLFloat(xmlFile, kommunalKey .. "#areaTrimmed")
	elseif hasXMLProperty(xmlFile, field_key) then -- maintain support for the previous save type
		self.area_tot = getXMLFloat(xmlFile, field_key .. "#areaToCut")
		self.area_trimmed = getXMLFloat(xmlFile, field_key .. "#areaTrimmed")
	end
	return true
end

function KommunalMission:saveToXMLFile(xmlFile, key) 
	self.fieldState = 2
	KommunalMission:superClass().saveToXMLFile(self, xmlFile, key)

	local kommunalKey = string.format("%s.kommunal", key)
	setXMLFloat(xmlFile, kommunalKey .. "#areaToCut", self.area_tot)
	setXMLFloat(xmlFile, kommunalKey .. "#areaTrimmed", self.area_trimmed)
end

function KommunalMission:init(field, ...)
	WeedMission:superClass().init(self, field, ...)
	self.fieldSpraySet = false
	local field_area = field.fieldArea
	local average_field_area = 0
	for field_idx=1, #g_fieldManager.fields do
		average_field_area = average_field_area + g_fieldManager.fields[field_idx].fieldArea
	end
	average_field_area = average_field_area / #g_fieldManager.fields

	self.area_tot = math.random()*400 + (1000 * field_area / average_field_area)
	self.rewardPerHa = 2*KommunalMission.REWARD_PER_HA * field_area/average_field_area
	self.field = field
	return true
end

-- non importa quale campo viene scelto per la missione, quindi lo accetto con una probabilità del 15% e finsice lì
function KommunalMission.canRunOnField(...)
	return true
end

-- chiamato quando clicchi sul contratto per ottenere le informazioni
function KommunalMission:getData()
	local area_truncated = tonumber(string.format("%.0f", self.area_tot))
	return {
		location = g_i18n:getText("fieldJob_location_kommunal"),
		jobType = g_i18n:getText("fieldJob_jobType_kommunal"),
		action = g_i18n:getText("fieldJob_desc_action_kommunal"),
		description = string.format(g_i18n:getText("fieldJob_desc_kommunal"), area_truncated)
	}
end

function KommunalMission:getCompletion()
	return math.min(1, self.area_trimmed/self.area_tot)
end

function KommunalMission:validate(event)
	return true
end

function KommunalMission:updateRewardPerHa()
	return nil
end

function KommunalMission:getVehicleVariant()
	return nil
end

function KommunalMission:resetField()
    return nil
end

function KommunalMission:completeField()
    return nil
end

function KommunalMission:registerTrimmedArea(area)
	self.area_trimmed = self.area_trimmed + area
end

function Contrastscheck()
	local bc_ModName = "FS22_BetterContracts"
	local bc_Ver = "1.2.8.8"
	if g_modIsLoaded[bc_ModName] then
		if g_modManager:getModByName(bcName).version < bc_Ver then 
			Logging.error("FS22_KommunalMission is incompatible with BetterContracts versions below %s! The mod will shut down.", bc_Ver)
			local info_diag_data = {text = string.format(g_i18n:getText("better_contracts_conflict"), bc_Ver)}
			g_gui:showInfoDialog(info_diag_data)
			return false
		end
	end 
	local cp_modName = "FS22_Contracts_Plus"
	if g_modIsLoaded[cp_modName] then
		Logging.error("FS22_KommunalMission is incompatible with Contracts plus! The mod will shut down.")
		local info_diag_data = {text = g_i18n:getText("contracts_plus_conflict")}
		g_gui:showInfoDialog(info_diag_data)
		return false
	end
	return true
end

if Contrastscheck() then
	-- register Kommunal Mission Type:
	g_missionManager:registerMissionType(KommunalMission, "kommunal")
end